
/* tune.c v0.0.5
 * This code demonstrates/exercises a new DVB API at:
 * http://linuxtv.org/hg/~stoth/s2
 *
 * The API enhances the current api, leaving all tuning mechanisms
 * untouched and fully working, but adds new tuning commands and modulation
 * types to allow new products to be added without having to revise the
 * kernel ABI in the future.
 *
 * This script demonstrates tuning traditional dvbs, dvbt or atsc products
 * with the new API (if people are curious about how they could migrate), but
 * more importantly it demonstrates the new modulation types and how they
 * should be tuned.
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/param.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>

#include <stdint.h>
#include <sys/time.h>

#include <linux/dvb/frontend.h>

#include <getopt.h>

/* http://www.stlinux.com/docs/manual/distribution/distribution_guide6.php */

/* Support for legacy us digital cable tuning, with the new API. */
struct dtv_property _qam_cmdargs[] = {
	{ .cmd = DTV_FREQUENCY,       .u.data = 579000000 },
	{ .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_BANDWIDTH,       .u.data = BANDWIDTH_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _qam_cmdseq = {
	.num = 5,
	.props = _qam_cmdargs
};

/* Support for 8PSK satellite tuning */
struct dtv_property _8psk_cmdargs[] = {
	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS2 },
	{ .cmd = DTV_FREQUENCY,       .u.data = 1451000 },
	{ .cmd = DTV_MODULATION,      .u.data = _8PSK },
	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
	{ .cmd = DTV_INNER_FEC,       .u.data = FEC_2_3 },
	{ .cmd = DTV_VOLTAGE,         .u.data = SEC_VOLTAGE_13 },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_PILOT,           .u.data = PILOT_AUTO },
	{ .cmd = DTV_ROLLOFF,         .u.data = ROLLOFF_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _8psk_cmdseq = {
	.num = 10,
	.props = _8psk_cmdargs
};

/* Support for legacy satellite tune, with the new API. */
struct dtv_property _dvbs_cmdargs[] = {
	{ .cmd = DTV_FREQUENCY,       .u.data = 1451000 },
	{ .cmd = DTV_MODULATION,      .u.data = QPSK },
	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
	{ .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
	{ .cmd = DTV_VOLTAGE,         .u.data = SEC_VOLTAGE_13 },
	{ .cmd = DTV_TONE,            .u.data = SEC_TONE_ON },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _dvbs_cmdseq = {
	.num = 8,
	.props = _dvbs_cmdargs
};

struct dtv_property _nbcqpsk_cmdargs[] = {
	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS2 },
	{ .cmd = DTV_FREQUENCY,       .u.data = 1451000 },
	{ .cmd = DTV_MODULATION,      .u.data = NBC_QPSK },
	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
	{ .cmd = DTV_INNER_FEC,       .u.data = FEC_9_10 },
	{ .cmd = DTV_VOLTAGE,         .u.data = SEC_VOLTAGE_13 },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_PILOT,           .u.data = PILOT_AUTO },
	{ .cmd = DTV_ROLLOFF,         .u.data = ROLLOFF_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _nbcqpsk_cmdseq = {
	.num = 10,
	.props = _nbcqpsk_cmdargs
};

/* Support for legacy dvb-t tune, with the new API. */
struct dtv_property _dvbt_cmdargs[] = {
	{ .cmd = DTV_FREQUENCY,       .u.data = 474000000 },
	{ .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_BANDWIDTH,       .u.data = BANDWIDTH_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _dvbt_cmdseq = {
	.num = 5,
	.props = _dvbt_cmdargs
};

/* 8/16 VSB tuning for legacy products, via the new API */
struct dtv_property _vsb_cmdargs[] = {
	{ .cmd = DTV_FREQUENCY,       .u.data = 479000000 },
	{ .cmd = DTV_MODULATION,      .u.data = VSB_8 },
	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _vsb_cmdseq = {
	.num = 4,
	.props = _vsb_cmdargs
};

/* Support for ISDB-T */
struct dtv_property _isdbt_cmdargs[] = {
	{ .cmd = DTV_DELIVERY_SYSTEM,    .u.data = SYS_ISDBT },
	{ .cmd = DTV_FREQUENCY,          .u.data = 474000000 },
	{ .cmd = DTV_MODULATION,         .u.data = QAM_AUTO },
	{ .cmd = DTV_INVERSION,          .u.data = INVERSION_AUTO },
	{ .cmd = DTV_ISDB_SEGMENT_IDX,   .u.data = 1 },
	{ .cmd = DTV_ISDB_SEGMENT_WIDTH, .u.data = 3 },
	{ .cmd = DTV_TUNE },
};
struct dtv_properties _isdbt_cmdseq = {
	.num = 7,
	.props = _isdbt_cmdargs
};

/* Used for queries */
struct dtv_property _delsys_cmdargs[] = {
	{ .cmd = DTV_DELIVERY_SYSTEM },
	{ .cmd = DTV_MODULATION },
};
struct dtv_properties _delsys_cmdseq = {
	.num = 2,
	.props = _delsys_cmdargs
};

void print_status(fe_status_t festatus)
{
	printf("FE_STATUS:");
	if (festatus & FE_HAS_SIGNAL) printf(" FE_HAS_SIGNAL");
	if (festatus & FE_TIMEDOUT) printf(" FE_TIMEDOUT");
	if (festatus & FE_HAS_LOCK) printf(" FE_HAS_LOCK");
	if (festatus & FE_HAS_CARRIER) printf(" FE_HAS_CARRIER");
	if (festatus & FE_HAS_VITERBI) printf(" FE_HAS_VITERBI");
	if (festatus & FE_HAS_SYNC) printf(" FE_HAS_SYNC");
	printf("\n");
}

void display_usage()
{
	printf("Usage:\n");
	printf(" -a# -- adapter no (default 0)\n");
	printf(" -f# -- frontend no (default 0)\n");
	printf(" -t# -- test to run (def 0=QAM, 1=VSB, 2=8PSK, 3=DVB-T\n");
	printf("                     4-ISDB-T, 5=DVBS, 6=NBC_QPSK)\n");
}

int main(int argc, char *argv[])
{
	int result, fefd = -1;
	fe_status_t festatus;
	int opt;
	int adapno = 0;
	int feno = 0;
	int testno = 0;
	char fname[256];
	struct dtv_properties *p = &_qam_cmdseq;

	if (argc == 1) {
		display_usage();
		exit(1);
	}

	while((opt = getopt(argc, argv, "a:f:t:")) != -1) {
		switch(opt) {
		case 'a':
			adapno = atoi(optarg);
			break;
		case 'f':
			feno = atoi(optarg);
			break;
		case 't':
			testno = atoi(optarg);
			if(testno == 0)
				p = &_qam_cmdseq;
			else
			if(testno == 1)
				p = &_vsb_cmdseq;
			else
			if(testno == 2)
				p = &_8psk_cmdseq;
			else
			if(testno == 3)
				p = &_dvbt_cmdseq;
			else
			if(testno == 4)
				p = &_isdbt_cmdseq;
			else
			if(testno == 5)
				p = &_dvbs_cmdseq;
			else
			if(testno == 6)
				p = &_nbcqpsk_cmdseq;
			break;
		case '?':
		case 'h':
			display_usage();
			exit(0);
		}
	}

	sprintf(fname, "/dev/dvb/adapter%d/frontend%d", adapno, feno);
	printf("%s\n", fname);
	printf("test %d\n", testno);

	if ((fefd = open(fname, O_RDWR | O_NONBLOCK)) < 0) {
		perror("open failed");
		return -1;
	}

	printf("ioctl result = %d\n", 
		ioctl(fefd, FE_SET_PROPERTY, p));

	while(1) {
		result = ioctl(fefd, FE_READ_STATUS, &festatus);
		printf("ioctl result = %d, [ 0x%02x ] = ", result, festatus);
		print_status(festatus);

		usleep(1 * 1000000);

		if (ioctl(fefd, FE_GET_PROPERTY, &_delsys_cmdseq) == 0) {
			printf("delivery_system = %d modulation = %d\n",
				_delsys_cmdargs[0].u.data,
				_delsys_cmdargs[1].u.data
				);
		}
	}
	
	close(fefd);

	return 0;
}

